package cz.muni.fi.pb162.collections;

import java.util.*;

/**
 * Trieda Collections ponúka škálu užitočných metód pre prácu s kolekciami
 * Ponúka len statické metódy a premenné, nie je z nej možné vytvoriť inštanciu (utility trieda)
 * V tomto deme nájdete ukážky najpoužívanejších metód z tejto triedy
 */
public class Demo {
    public static void main(String[] args) {
        Set<String> numbers = Set.of("One", "Two", "Three");
        numbers.add("Zdar");

        // ak chceme kolekciu vrátiť ako návratovú hodnotu metódy, je vhodné vrátiť nemodifikovateľnú kolekciu
        // vieme využiť metódu unmodifiableSet/unmodifiableList/unmodifiableCollection
        // táto metóda vracia nemodifikovateľnú kópiu našej kolekcie
        Set<String> unmodifiableNumbers = Collections.unmodifiableSet(numbers);
        System.out.println("Vytvorili sme kópiu množiny " + numbers + ", ktorá je nemodifikovateľná: " + unmodifiableNumbers);

        // volanie metódy add na nemodifikovateľnej kolekcii vyvolá runtime výnimku
        // unmodifiableNumbers.add("Four"); // po odkomentovaní padne program za behu

        // ak niekde potrebujem využiť prázdnu kolekciu vieme využiť metódu emptyList()/emptySet()...
        // vrátená prázdna kolekcia je nemodifikovateľná
        Set<String> emptyCollection = Collections.emptySet();
        System.out.println("Veľkosť prázdnej kolekcie získanej metódou emptySet() je: " + emptyCollection.size());
        // emptyCollection.add("Something"); // po odkomentovaní padne program za behu

        List<String> words = new ArrayList<>(List.of("Car", "Door", "Avocado"));

        // ak chceme zoradiť prvky v zozname vieme využiť metódu sort(List<Type> list)
        // typ objektov v zozname musí mať definované prirodzené usporiadanie
        // ak nemá definované alebo chceme iné usporiadanie musíme vložiť Comparator ako druhý parameter metódy sort
        // viac o usporiadaní nájdete v balíčku comparable
        System.out.println("Zoznam pred usporiadaním: " + words);
        Collections.sort(words);
        System.out.println("Zoznam po usporiadaní: " + words);

        // ak chceme využiť binárne vyhľadávanie môžeme využiť metódu binarySearch
        // tá vráti index objektu, ktorý hľadáme alebo záporné číslo ak sa tám daný kľúč nenachádza
        // podmienkou je aby bol zoznam zoradený, na nezoradenom zozname binary search nebude fungovať správne
        System.out.println("Index prvku \"Avocado\" v zozname " + words + " je " + Collections.binarySearch(words, "Avocado"));
        System.out.println("Index prvku \"Beta\" v zozname " + words + " je " + Collections.binarySearch(words, "Beta"));

        // ak chceme vymeniť pozície dvoch prvkov v kolekcii, môžeme využiť metódu swap(Collection collection, int index1, int index2)
        // ak by bol index1 alebo index2 mimo rozsah kolekciie(< 0 alebo >= size()), metóda vyhodí výnimku
        System.out.println("Zoznam words pred výmenou: " + words);
        Collections.swap(words, 0, 2);
        System.out.println("Zoznam words po výmene: " + words);

        // ak chceme náhodne premiešať prvky v zozname, môžeme využiť metódu shuffle(List list)
        System.out.println("Zoznam words pred premiešaním: " + words);
        Collections.shuffle(words);
        System.out.println("Zoznam words po premiešaní: " + words);

        // ak chceme obrátiť poradie prvkov v zozname, môžeme využiť metódu reverse(List list)
        System.out.println("Zoznam words pred obrátením: " + words);
        Collections.reverse(words);
        System.out.println("Zoznam words po obrátení: " + words);

        // ak chceme získať minimálnu, poprípade maximálnu hodnotu v kolekcii vieme využiť metódu min/max
        // typ prvkov v kolekcii musí mať zadefinované prirodzené usporiadanie
        System.out.println("Minimálny prvok v kolekcii je: " + Collections.min(words));
        System.out.println("Maximálny prvok v kolekcii je: " + Collections.max(words));

        words.add("Avocado");
        // ak chceme zisť koľko krát sa prvok nachádza v kolekcii(kardinalitu), vieme využiť metódu frequency
        System.out.println("Prvok \"Tree\" sa nachádza v zozname " + words + ": " + Collections.frequency(words, "Tree") + " krát");
        System.out.println("Prvok \"Avocado\" sa nachádza v zozname " + words + ": " + Collections.frequency(words, "Avocado") + " krát");
    }
}
