package cz.muni.fi.pb162.comparable;

import java.util.Objects;

/**
 * Trieda Car predstavuje triedu s prirodzeným usporiadaním
 * Trieda Car bude prirodzene usporiadaná abecedne podľa ŠPZ vozidla
 */
public class Car implements Comparable<Car>{
    private final String brand;
    private final String model;
    private final int yearOfProduction;
    private final String licensePlate;

    public Car(String brand, String model, int yearOfProduction, String licensePlate) {
        this.brand = brand;
        this.model = model;
        this.yearOfProduction = yearOfProduction;
        this.licensePlate = licensePlate;
    }

    public String getBrand() {
        return brand;
    }

    public String getModel() {
        return model;
    }

    public int getYearOfProduction() {
        return yearOfProduction;
    }

    public String getLicensePlate() {
        return licensePlate;
    }

    // ak metóda equals vráti hodnotu true, tak by metóda compareTo mala vrátiť hodnotu 0
    // nie je to však nutné (napríklad rovnosť založená na značke automobilu, prirodzené usporiadanie podľa roku výroby)
    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (!(o instanceof Car)) return false;
        Car car = (Car) o;
        return Objects.equals(licensePlate, car.licensePlate);
    }

    @Override
    public int hashCode() {
        return Objects.hash(licensePlate);
    }

    @Override
    public String toString() {
        return brand + " " + model + " (" + yearOfProduction + "): " + licensePlate;
    }

    // porovnáme jednotlivé ŠPZ metódou compareTo a vrátime jej výsledok
    // keďže je ŠPZ typu String, zavolá sa metóda compareTo z triedy String
    // Prirodzené usporiadanie triedy String je abecedné (auto < dvere)
    // metóda compareTo môže vrátiť tri rôzne výsledky:
    //      1. záporný ak this.licensePlate je v abecede skôr ako car.licensePlate
    //      2. 0 ak sú licensePlate rovnaké
    //      3. kladný ak this.licensePlate je v abecede neskôr ako car.licensePlate
    @Override
    public int compareTo(Car car) {
        return licensePlate.compareTo(car.getLicensePlate());
    }
}
