package cz.muni.fi.pb162.comparable;

import java.util.List;
import java.util.Set;
import java.util.TreeSet;

/**
 * Java ponúka dve rozhrania určené na zoraďovanie objektov podľa atribútov triedy
 *      1. Comparable - rozširuje priamo triedu, ktorej objekty chceme porovnávať
 *                    - slúži na definovanie prirodzeného usporiadania triedy
 *                    - využité najmä v kombinácii s usporiadanými kolekciami
 *                    - predpisuje jedinú metódu compareTo(T object)
 *                    - ukážka triedy implementujúca rozhranie Comparable je trieda Car v tomto balíku
 *
 *      2. Comparator - slúži na porovanie zvonku pomocou objektu inej triedy
 *                    - predpisuje jedinú metódu compare(T o1, T o2), kde T je typ triedy, ktorej objekty chceme porovnávať
 *                    - funguje ako alternatíva ak chceme využiť iné ako prirodzené usporiadanie
 */
public class Demo {
    public static void main(String[] args) {
        List<Car> cars = List.of(new Car("Ford", "Focus", 2006, "ABC123"),
                new Car("Audi", "Quattro", 1990, "XYZ456"),
                new Car("Subaru", "BRZ", 2016, "KRE198"));

        // použitím usporiadanej kolekcie sa automaticky využíva prirodzené usporiadanie zadefinované nad triedou Car
        Set<Car> naturallySortedCars = new TreeSet<>();
        naturallySortedCars.addAll(cars);
        System.out.println("Autá zoradené prirodzene podľa ŠPZ: " + naturallySortedCars);

        // ak chceme aby bola trieda usporiadaná inak vložíme do konštruktoru Comparator
        // Comparator je inštanciou triedy implementujúcej rozhranie Comparator<comparedClass> (comparedClass je typ objektov, ktoré porovnávame)
        // comparedClass musí byť rovnaká ako typ objektov vkladaných do kolekcie
        // v tomto prípade Set<Car> a Comparator<Car>
        Set<Car> sortedWithComparator = new TreeSet<>(new CarProductionYearComparator());
        sortedWithComparator.addAll(cars);
        System.out.println("Autá zoradené pomocou comparátoru podľa roku výroby: " + sortedWithComparator);
    }
}
