package cz.muni.fi.pb162.map;

import java.util.HashMap;
import java.util.Map;

/**
 * Mapa je dynamická dátová Štruktúra, ktorá ukladá dvojicu - kľúč a hodnotu
 * Umožňuje rýchle vyhľadávanie podľa kľúča
 * Kľúče v mape sú vždy unikátne
 * Medzi najpoužívanejšie implementácie rozhrania Map patria triedy HashMap a TreeMap (usporiadané kľúče)
 *
 * V tomto deme nájdete ukážky najpoužívanejších metód
 */
public class Demo {

    public static void main(String[] args) {

        Map<Integer, String> numbers = new HashMap<>();

        // na pridanie nového prvku do mapy využijeme metódu put(key, value)
        numbers.put(1, "One");
        numbers.put(2, "Two");
        numbers.put(3, "Four");

        // ak pridáme dvojicu s kľúčom, ktorý sa už v mape nachádza pôvodná hodnota je nahradená novou
        // metóda vracia pôvodnu hodnotu
        System.out.println("Ukladám do mapy " + numbers + " hodnotu \"Three\" s kľúčom \"1\"");
        System.out.println("Pôvodná hodnota v mape pre kľúč \"3\" bola: " + numbers.put(3, "Three"));
        System.out.println("Mapa number: " + numbers);

        // na zistenie počtu prvkov v mape použijeme metódu size()
        System.out.println("Počet prvkov v mape " + numbers + " je: " + numbers.size());

        // na zistenie či sa daný kľúč nachádza v mape použijeme metódu containsKey(key)
        System.out.println("Obsahuje mapa " + numbers + " kľúč \"1\": " + numbers.containsKey(1));

        // na zistenie či sa daná hodnota nachádza v mape použijeme metódu containsValue(value)
        System.out.println("Obsahuje mapa " + numbers + " hodnotu \"Four\": " + numbers.containsValue("Four"));

        // na nahradenie hodnoty pre nejaký kľúč môžeme využiť metódu replace(key, value)
        // na rozdiel od metódy put(key, value) sa do mapy nič nepridá ak tam daný kľúč nie je
        System.out.println("Nahradzujem hodnotu kľúču \"1\" za \"Uno\"");
        numbers.replace(1, "Uno");
        System.out.println("Mapa number: " + numbers);

        // ak chceme vložiť prvok do mapy len v prípade, že sa ešte v mape nenachádza použijeme metódu putIfAbsent(key, value)
        System.out.println("Vkládam do mapy " + numbers + " kľúč \"1\" a hodnotu \"One\" ak sa tam nenachádza");
        numbers.putIfAbsent(1, "One"); // kľúč nie je nahradený
        System.out.println("Mapa number: " + numbers);

        // na získanie prvku pomocou kľúču použijeme metódu get(key)
        System.out.println("V mape " + numbers + " má prvok s kľúčom \"2\" hodnotu: " + numbers.get(2));

        // ak chceme získať všetky kľúče uložené v mape použijeme metódu keySet()
        System.out.println("Všetky kľúče mapy " + numbers + " sú: " + numbers.keySet());

        // ak chceme získať všetky hodnoty uložené v mape použijeme metódu values()
        System.out.println("Všetky hodnoty mapy " + numbers + " sú: " + numbers.values());

        // ak chceme iterovať jednotlivými dvojicami v mape využijeme for cyklus a metódu entrySet()
        // jednotlivé prvky sú typu Entry<keyType, valueType>
        System.out.println("Iterujem jednotlivými dvojicami mapy " + numbers);
        for(Map.Entry<Integer, String> entry : numbers.entrySet()) {
            System.out.println("Key: " + entry.getKey());
            System.out.println("Value: " + entry.getValue());
            System.out.println();
        }

        // na vymazanie celého obsahu mapy vieme využiť metódu clear()
        System.out.println("Vymazávam všetky prvky z mapy " + numbers);
        numbers.clear();
        System.out.println("Mapa number: " + numbers);
    }
}
